/*-
 * Copyright (c) 2012,2013 Darran Hunt (darran [at] hunt dot net dot nz)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES,
 * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 * THE CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/**
 * @file WiFly RN-XV Library
 */



//############################################################################
//# HISTORY  PKT_WiFlyHQ.c
//#
//# noch UNVOLLSTNDIG!! (Cebra)
//#
//# 01.04.2014
//# - chg: Nicht bentigte Progmem Strings auskommentiert
//#
//# 11.03.2014 SH
//# - add: neue Funktionen wl_reset() und wl_update()
//# - chg: wl_init() wurde komplett auf den neuen AP Modus gendert, die Init Routine vom Ad Hoc Modus befindet sich ausgeklammert darunter
//# - chg: getVersion wurde angepasst + neuer Wert fr resp_Version[] PROGMEM = "RN-171\r\n" anstatt WiFly
//# - chg: setSSID() und setPassphrase() wurden um weiteren Parameter ergnzt
//# - chg: createAdhocNetwork angepasst da Funktion setSSID() enthalten
//# - del: ausgeklammerte Funktionen getConnection, getDHCPMode, getSpaceReplace, getopt_Asc und join
//# 
//# 12.02.2014 OG
//# - chg: readTimeout() Auskommentiert: "unused variable 'ind'"
//# - chg: wl_init() Auskommentiert: "unused variable 'dhcpMode'"
//#
//# 02.07.2013 cebra
//# - new: Routinen fr WiFly WLAN-Modul
//#
//###########################################################################


#include <avr/pgmspace.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdarg.h>
#include <util/delay.h>
#include <inttypes.h>
#include <string.h>
#include "../main.h"
#include "PKT_WiFlyHQ.h"
#include "../eeprom/eeprom.h"
#include "../uart/uart1.h"
#include "../timer/timer.h"
#include "../lcd/lcd.h"

#ifdef USE_WLAN

/* Change these to match your WiFi network */
const char mySSID[] = "PKT";
const char myPassword[] = "123456";

#define WIFLY_STATUS_TCP_MASK 		0x000F
#define WIFLY_STATUS_TCP_OFFSET		0
#define WIFLY_STATUS_ASSOC_MASK 	0x0001
#define WIFLY_STATUS_ASSOC_OFFSET	4
#define WIFLY_STATUS_AUTHEN_MASK 	0x0001
#define WIFLY_STATUS_AUTHEN_OFFSET	5
#define WIFLY_STATUS_DNS_SERVER_MASK	0x0001
#define WIFLY_STATUS_DNS_SERVER_OFFSET	6
#define WIFLY_STATUS_DNS_FOUND_MASK	0x0001
#define WIFLY_STATUS_DNS_FOUND_OFFSET	7
#define WIFLY_STATUS_CHAN_MASK 		0x000F
#define WIFLY_STATUS_CHAN_OFFSET	9

#define WIFLY_TCP_IDLE		0
#define WIFLY_TCP_CONNECTED	1
#define WIFLY_TCP_NOIP		3
#define WIFLY_TCP_CONNECTING	4

/* WiFi data rates */
#define WIFLY_RATE_1MBPS	0
#define WIFLY_RATE_2MBPS	1
#define WIFLY_RATE_5_5MBPS	2	/* 5.5 MBps */
#define WIFLY_RATE_6MBPS	8
#define WIFLY_RATE_9MBPS	9
#define WIFLY_RATE_11MBPS	3
#define WIFLY_RATE_12MBPS	10
#define WIFLY_RATE_18MBPS	11
#define WIFLY_RATE_24MBPS	12	/* Default */
#define WIFLY_RATE_36MBPS	13
#define WIFLY_RATE_48MBPS	14
#define WIFLY_RATE_54MBPS	15

/* Request and response strings in PROGMEM */
/*const char req_GetIP[] PROGMEM = "get ip\r";
const char resp_IP[] PROGMEM = "IP=";
const char resp_NM[] PROGMEM = "NM=";
const char resp_GW[] PROGMEM = "GW=";
const char resp_Host[] PROGMEM = "HOST=";
const char resp_DHCP[] PROGMEM = "DHCP=";
const char req_GetMAC[] PROGMEM = "get mac\r";
const char resp_MAC[] PROGMEM = "Mac Addr=";
const char req_GetWLAN[] PROGMEM = "get wlan\r";
const char resp_SSID[] PROGMEM = "SSID=";
const char resp_Chan[] PROGMEM = "Chan=";
const char req_GetOpt[] PROGMEM = "get opt\r";
const char resp_DeviceID[] PROGMEM = "DeviceId=";
const char req_GetUart[] PROGMEM = "get u\r";
const char resp_Baud[] PROGMEM = "Baudrate=";
const char req_GetTime[] PROGMEM = "get time\r";
const char resp_Zone[] PROGMEM = "Zone=";
const char req_ShowTime[] PROGMEM = "show time\r";
const char resp_Uptime[] PROGMEM = "UpTime=";
const char resp_Time[] PROGMEM = "Time=";
const char req_GetDNS[] PROGMEM = "get dns\r";
const char resp_DNSAddr[] PROGMEM = "Address=";
const char req_ShowTimeT[] PROGMEM = "show t t\r";
const char resp_RTC[] PROGMEM = "RTC=";
const char resp_Mode[] PROGMEM = "Mode=";
const char req_GetComm[] PROGMEM = "get comm\r";
const char resp_FlushTimeout[] PROGMEM = "FlushTimer=";
const char resp_FlushChar[] PROGMEM = "MatchChar=";
const char resp_FlushSize[] PROGMEM = "FlushSize=";
const char req_GetRSSI[] PROGMEM = "show rssi\r";
const char resp_RSSI[] PROGMEM = "RSSI=(-";
const char resp_Flags[] PROGMEM = "FLAGS=0x";
const char resp_Protocol[] PROGMEM = "PROTO=";
const char req_GetAdhoc[] PROGMEM = "get adhoc\r";
const char resp_Beacon[] PROGMEM = "Beacon=";
const char resp_Probe[] PROGMEM = "Probe=";
const char resp_Reboot[] PROGMEM = "Reboot=";
const char resp_Join[] PROGMEM = "Join=";
const char resp_Rate[] PROGMEM = "Rate=";
const char resp_Power[] PROGMEM = "TxPower=";
const char resp_Replace[] PROGMEM = "Replace=";*/
const char req_GetVersion[] PROGMEM = "ver\r";
const char resp_Version[] PROGMEM = "RN-171\r\n"; //WiFly


/* Request and response for specific info */
static struct 
{
    const char *req;
    const char *resp;
} requests[] = {
    /*{ req_GetIP,        resp_IP },       // 0
    { req_GetIP,        resp_NM },       // 1
    { req_GetIP,        resp_GW },       // 2
    { req_GetMAC,       resp_MAC },      // 3
    { req_GetWLAN,      resp_SSID },     // 4
    { req_GetOpt,       resp_DeviceID }, // 5
    { req_GetUart,      resp_Baud },     // 6
    { req_ShowTime,     resp_Time },     // 7
    { req_ShowTime,     resp_Uptime },   // 8
    { req_GetTime,      resp_Zone },     // 9
    { req_GetDNS,       resp_DNSAddr },  // 10
    { req_ShowTimeT,    resp_RTC },      // 11
    { req_GetIP,        resp_DHCP },     // 12
    { req_GetUart,      resp_Mode },     // 13
    { req_GetComm,      resp_FlushTimeout }, // 14
    { req_GetComm,      resp_FlushChar }, // 15
    { req_GetComm,      resp_FlushSize }, // 16
    { req_GetRSSI,      resp_RSSI },     // 17
    { req_GetIP,        resp_Flags },    // 18
    { req_GetIP,        resp_Host },     // 19
    { req_GetIP,        resp_Protocol }, // 20
    { req_GetAdhoc,     resp_Beacon },   // 21
    { req_GetAdhoc,     resp_Probe },    // 22
    { req_GetAdhoc,     resp_Reboot },   // 23
    { req_GetWLAN,      resp_Join },     // 24
    { req_GetWLAN,      resp_Rate },     // 25
    { req_GetWLAN,      resp_Power },    // 26
    { req_GetOpt,       resp_Replace },*/  // 27
    { req_GetVersion,   resp_Version },  // 28
};

/* Request indices, must match table above */
typedef enum {
    /*WIFLY_GET_IP        = 0,
    WIFLY_GET_NETMASK   = 1,
    WIFLY_GET_GATEWAY   = 2,
    WIFLY_GET_MAC       = 3,
    WIFLY_GET_SSID      = 4,
    WIFLY_GET_DEVICEID  = 5,
    WIFLY_GET_BAUD      = 6,
    WIFLY_GET_TIME      = 7,
    WIFLY_GET_UPTIME    = 8,
    WIFLY_GET_ZONE      = 9,
    WIFLY_GET_DNS       = 10,
    WIFLY_GET_RTC       = 11,
    WIFLY_GET_DHCP      = 12,
    WIFLY_GET_UART_MODE = 13,
    WIFLY_GET_FLUSHTIMEOUT = 14,
    WIFLY_GET_FLUSHCHAR = 15,
    WIFLY_GET_FLUSHSIZE = 16,
    WIFLY_GET_RSSI      = 17,
    WIFLY_GET_IP_FLAGS  = 18,
    WIFLY_GET_HOST      = 19,
    WIFLY_GET_PROTOCOL  = 20,
    WIFLY_GET_BEACON    = 21,
    WIFLY_GET_PROBE     = 22,
    WIFLY_GET_REBOOT    = 23,
    WIFLY_GET_JOIN      = 24,
    WIFLY_GET_RATE      = 25,
    WIFLY_GET_POWER     = 26,
    WIFLY_GET_REPLACE   = 27,*/
    WIFLY_GET_VERSION   = 0, //28
} e_wifly_requests;

bool inCommandMode;
int  exitCommand;
bool dhcp;
bool restoreHost;
bool restoreHostStored;
char lastHost[32];
uint16_t lastPort;

bool tcpMode;
bool udpAutoPair;

bool tcpMode;
bool udpAutoPair;

bool connected;
bool connecting;

bool debugOn;

char buf[80];

uint8_t Result;
uint8_t Old_Baudrate;

/*  for dbgDump() */
char *dbgBuf;
int dbgInd;
int dbgMax;

char replaceChar;      /* The space replacement character */
static char prompt[16];
static bool gotPrompt = false;


/**
 * Convert a unsigned int to a string
 * @param val the value to convert to a string
 * @param base format for string; either DEC for decimal or
 *             HEX for hexidecimal
 * @param buf the buffer to write the string to
 * @param size the size of the buffer
 * @returns number of characters written to the buffer
 *          not including the null terminator (i.e. size of the string)
 **/
int simple_utoa(uint32_t val, uint8_t base, char *buf, int size)
{
	char tmpbuf[16];
    int ind=0;
    uint32_t nval;
    int fsize=0;

    if (base == DEC) 
	{
        do 
		{
            nval = val / 10;
            tmpbuf[ind++] = '0' + val - (nval * 10);
            val = nval;
        } while (val);
    } 
	else 
	{
		do 
		{
            nval = val & 0x0F;
            tmpbuf[ind++] = nval + ((nval < 10) ? '0' : 'A');
            val >>= 4;
        } while (val);
        tmpbuf[ind++] = 'x';
        tmpbuf[ind++] = '0';
    }

    ind--;

    do 
	{
        buf[fsize++] = tmpbuf[ind];
    } while ((ind-- > 0) && (fsize < (size-1)));
	
    buf[fsize] = '\0';
	
    return (fsize);
}


/** Simple hex string to uint32_t */
uint32_t atoh(char *buf)
{
    uint32_t res=0;
    char ch;
    bool gotX = false;

    while ((ch=*buf++) != 0) 
	{
        if (ch >= '0' && ch <= '9') res = (res << 4) + ch - '0';
		else if (ch >= 'a' && ch <= 'f') res = (res << 4) + ch - 'a' + 10;
		else if (ch >= 'A' && ch <= 'F') res = (res << 4) + ch - 'A' + 10;
		else if ((ch == 'x') && !gotX) gotX = true; /* Ignore 0x at start */
		else break;
    }

    return (res);
}


/** Simple ASCII to unsigned int */
uint32_t atou(const char *buf)
{
    uint32_t res=0;

    while (*buf) 
	{
        if ((*buf < '0') || (*buf > '9')) break;
        res = res * 10 + *buf - '0';
        buf++;
    }

    return (res);
}


/** Send a string to the WiFly */
void sends(const char *str)
{
    uart1_puts(str);
}


/** Send a character to the WiFly */
void sendc(const char ch)
{
    uart1_putc(ch);
}


/** Send a string from PROGMEM to the WiFly */
void send_P(const char *str)
{
    uart1_puts_p(str);
}


/**
 * Flush the incoming data from the WiFly.
 * @param timeout - the number of milliseconds to wait for additional data to flush. Default is 500msecs.
 */
void flushRx(int timeout)
{
    char ch;
    while (readTimeout(&ch,timeout));
}


/**
 * Write a byte to the WiFly.
 * @param byte - the byte to write.
 * @return the number of bytes written (1).
 */
uint8_t write(uint8_t byte)
{
    if (dbgInd < dbgMax) dbgBuf[dbgInd++] = byte;
    uart1_putc(byte);
	
    return (1);
}


/** Read the next character from the WiFly serial interface.
 * Waits up to timeout milliseconds to receive the character.
 * @param chp pointer to store the read character in
 * @param timeout the number of milliseconds to wait for a character
 * @retval true - character read
 * @retval false - timeout reached, character not read
 */
bool readTimeout(char *chp, uint16_t timeout)
{
    char ch;
    timer2 = timeout;

    while (timer2>0)
    {
		if (uart1_available() > 0) 
		{
            ch = uart1_getc();
            *chp = ch;

            return (true);
        }
    }
	
    return (false);
}


/** Scan the input data for the WiFLy prompt.  This is a string starting with a '<' and
 * ending with a '>'. Store the prompt for future use.
 */
bool setPrompt()
{
    char ch;

    while (readTimeout(&ch,WIFLY_DEFAULT_TIMEOUT)) 
	{
        if (ch == '<') 
		{
            uint8_t ind = 1;
            prompt[0] = ch;
            while (ind < (sizeof(prompt)-4)) 
			{
                if (readTimeout(&ch,WIFLY_DEFAULT_TIMEOUT)) 
				{
                    prompt[ind++] = ch;
                    if (ch == '>') 
					{
                        if (readTimeout(&ch,WIFLY_DEFAULT_TIMEOUT)) 
						{
                            if (ch == ' ') 
							{
                                prompt[ind++] = ch;
                                prompt[ind] = 0;
                                gotPrompt = true;
                                gets(NULL,0,WIFLY_DEFAULT_TIMEOUT);
                                return true;
                            } 
							else return (false); /* wrong character */
                        } 
						else return (false); /* timeout */
                    }
                } 
				else return (false); /* timeout */
            }
			
            return (false);
        }
    }

    return (false);
}


/**
 * Read characters from the WiFly until the prompt string is seen.
 * Characters are discarded until the prompt is found.
 * Fails if no data is received for the timeout duration.
 * @timeout fail if no data received for this period (in milliseconds).
 * @retval true - prompt found
 * @retval false - prompt not found, timeout reached
 * @Note: The first time this function is called it will set the prompt string.
 */
bool getPrompt(uint16_t timeout)
{
    bool res;

    if (!gotPrompt)  res = setPrompt(); 
	else res = match(prompt, timeout);
	
    return (res);
}


/**
 * Read characters from the WiFly and match them against the
 * progmem string. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until the string is matched
 * or until no characters are received for the timeout duration.
 * @param str The string to match, in progmem.
 * @param timeout fail if no data received for this period (in milliseconds).
 * @retval true - a match was found
 * @retval false - no match found, timeout reached
 */
bool match_P(const char *str, uint16_t timeout)
{
    const char *match = str;
    char ch, ch_P;

    ch_P = pgm_read_byte(match);
    if (ch_P == '\0') return true; /* Null string always matches */

    while (readTimeout(&ch,timeout)) 
	{
        if (ch == ch_P) match++;
		else 
		{
            match = str; /* Restart match */
            if (ch == pgm_read_byte(match)) match++;
        }

        ch_P = pgm_read_byte(match);
        if (ch_P == '\0') return (true);
    }

    return (false);
}



/** Put the WiFly into command mode */
bool enterCommandMode(void)
{
    uint8_t retry;

    if (inCommandMode) return (true);

    _delay_ms(250);
    send_P(PSTR("$$$"));
    _delay_ms(250);
	
    if (match_P(PSTR("CMD\r\n"), WIFLY_COMMAND_TIMEOUT)) 
	{
        /* Get the prompt */
        if (gotPrompt) 
		{
            inCommandMode = true;
            return (true);
        } 
		else 
		{
            for (retry=0; retry < 2; retry++) 
			{
                uart1_putc('\r');

                if (getPrompt(WIFLY_COMMAND_TIMEOUT)) 
				{
                    inCommandMode = true;
                    return (true);
                }
            }
        }
    }
    /* See if we're already in command mode */
    uart1_putc('\r');
    if (getPrompt(WIFLY_COMMAND_TIMEOUT)) 
	{
        inCommandMode = true;
        return (true);
    }

    for (retry=0; retry<2; retry++) 
	{
        _delay_ms(250);
        send_P(PSTR("$$$"));
        _delay_ms(250);
		
        if (match_P(PSTR("CMD\r\n"), WIFLY_COMMAND_TIMEOUT)) 
		{
            inCommandMode = true;
            return (true);
        }
    }

    return (false);
}


/* Get the WiFly ready to receive a command. */
bool startCommand(void)
{
    if (!inCommandMode) 
	{
        if (!enterCommandMode()) return false;
        /* If we're already in command mode, then we don't exit it in finishCommand().
         * This is an optimisation to avoid switching in and out of command mode
         * when using several commands to implement another command.
         */
    } 
    exitCommand++;
	
    return (true);
}


/**
* Get WiFly ready to handle commands, and determine
* some initial status.
*/
uint8_t wl_init(void) //AP Mode
{
	uint8_t result = true;

	inCommandMode = false;
	exitCommand = 0;
	connected = false;
	connecting = false;
	dhcp = true;
	restoreHost = true;
	dbgBuf = NULL;
	dbgInd = 0;
	dbgMax = 0;
	lastPort = 0;
	lastHost[0] = 0;

	set_Modul_On(Wlan);
	_delay_ms(4000);

	if (!find_WiFly()) 
	{
		lcd_printp_at( 0, 5, PSTR("WiFly not found"), MNORMAL);
		set_Modul_On(USB);
		return (false);
	}
	else lcd_printp_at( 0, 5, PSTR("WiFly found"), MNORMAL);
	
	setopt_RM(PSTR("set u b 57600"), (char *)NULL, (char *)NULL,false);	
	
	_delay_ms(1000);
	save();
	_delay_ms(1000);
	reboot();

	SetBaudUart1(Config.PKT_Baudrate);

	if (!setopt_RM(PSTR("set wlan join 7"), (char *)NULL, (char *)NULL,false)) //enables Access Point mode
	{
		lcd_printp_at( 0, 6, PSTR("F set wlan join"), MNORMAL);
		result = false;
	}

	if (!setopt_RM(PSTR("set ip dhcp 4"), (char *)NULL, (char *)NULL,false)) //enables DHCP server
	{
		lcd_printp_at( 0, 6, PSTR("F set ip dhcp"), MNORMAL);
		result = false;
	}
		
	if (!setopt_RM(PSTR("set ip address 169.254.1.1"), (char *)NULL, (char *)NULL,false)) //set IP address
	{
		lcd_printp_at( 0, 6, PSTR("F ip address"), MNORMAL);
		result = false;
	}
		
	if (!setopt_RM(PSTR("set ip gateway 169.254.1.1"), (char *)NULL, (char *)NULL,false)) //for Apple iOS needs to be the same as the IP
	{
		lcd_printp_at( 0, 6, PSTR("F set ip gateway"), MNORMAL);
		result = false;
	}
		
	if (!setopt_RM(PSTR("set ip netmask 255.255.255.0"), (char *)NULL, (char *)NULL,false)) //set IP netmask
	{
		lcd_printp_at( 0, 6, PSTR("F set ip netmask"), MNORMAL);
		result = false;
	}
	
	if (!setopt_RM(PSTR("set wlan rate 3"), (char *)NULL, (char *)NULL,false)) //set Wlan rate, value between 0 - 12
	{
		lcd_printp_at( 0, 6, PSTR("F wlan rate"), MNORMAL);
		result = false;
	}
		
	if (!setSSID(Config.Wlan_SSID, true)) //set ssid 
	{
		lcd_printp_at( 0, 6, PSTR("F set apmode ssid"), MNORMAL);
		result = false;
	}
		
	if (!setPassphrase(Config.Wlan_Password, true)) //set passphrase, least 8 characters long, max 32
	{
		lcd_printp_at( 0, 6, PSTR("F apmode pass"), MNORMAL);
		result = false;
	}
		
	if (!setChannel(Config.Wlan_Channel)) //set Wlan channel
	{			
		lcd_printp_at( 0, 6, PSTR("F wlan channel"), MNORMAL);
		result = false;
	}		
		
	if (!setopt_RM(PSTR("set ip protocol 0x3"), (char *)NULL, (char *)NULL,false)) //enables UDP connections mode
	{
		lcd_printp_at( 0, 6, PSTR("F set ip protocol"), MNORMAL);
		result = false;
	}
		
	if (!setopt_RM(PSTR("set apmode link_monitor 0"), (char *)NULL, (char *)NULL,false)) //disables autho deauthentification of clients
	{
		lcd_printp_at( 0, 6, PSTR("F apmode link_monitor"), MNORMAL);
		result = false;
	}
		
	_delay_ms(1000);
	save();
	_delay_ms(1000);
	reboot();
	
	set_Modul_On(USB);
	if (!result) _delay_ms(4000);
	
	return (result);
}

/*uint8_t wl_init(void) //AdHoc Mode
{
	//int8_t dhcpMode=0;
	uint8_t result = true;

	inCommandMode = false;
	exitCommand = 0;
	connected = false;
	connecting = false;
	dhcp = true;
	restoreHost = true;
	#ifdef DEBUG
	debugOn = true;
	#else
	debugOn = false;
	#endif

	dbgBuf = NULL;
	dbgInd = 0;
	dbgMax = 0;
	lastPort = 0;
	lastHost[0] = 0;


	set_Modul_On(Wlan);
	_delay_ms(4000);

	if (!find_WiFly()) 
	{
		lcd_printp_at( 0, 2, PSTR("WiFly not found"), MNORMAL);
		return (false);
	}

	if(!setBaud(Config.PKT_Baudrate))
	{
		lcd_printp_at( 0, 2, PSTR("Error SetBaudrate"), MNORMAL);
		SetBaudUart1(Config.PKT_Baudrate);
		result = false;
	}

	SetBaudUart1(Config.PKT_Baudrate);

	if (result)
	{
		if (!setopt_RM(PSTR("set u m 1"), (char *)NULL, (char *)NULL,false)) 
		{
			lcd_printp_at( 0, 3, PSTR("F turn off echo"), MNORMAL);
			result = false;
		}

		if (!setopt_RM(PSTR("set comm remote 0"), (char *)NULL, (char *)NULL,false)) 
		{
			lcd_printp_at( 0, 5, PSTR("F set comm rem"), MNORMAL);
			result = false;
		}

		if (!Config.Wlan_WPA)
		{
			lcd_printp_at( 0, 4, PSTR("Init Adhoc Network"), MNORMAL);
			if (createAdhocNetwork(Config.Wlan_SSID, Config.Wlan_Channel)>0)
			{
				lcd_printp_at( 0, 4, PSTR("Fail Init Adhoc "), MNORMAL);
				lcd_print_hex(Result,0);
				result = false;
			}
		}
	}

	set_Modul_On(USB);
	if (!result) _delay_ms(4000);
	
	return (result);
}*/

uint8_t wl_reset(void)
{	
	set_Modul_On(Wlan);
	_delay_ms(4000);

	if (!find_WiFly())
	{
		lcd_printp_at( 0, 5, PSTR("WiFly not found"), MNORMAL);
		set_Modul_On(USB);
		return (false);
	}
	else lcd_printp_at( 0, 5, PSTR("WiFly found"), MNORMAL);
	
	setopt_RM(PSTR("factory RESET"), (char *)NULL, (char *)NULL,false);
	_delay_ms(1000);
	save();
	_delay_ms(1000);
	reboot();
	
	set_Modul_On(USB);
	
	return (true);
}

uint8_t wl_update(void)
{
	uint8_t result = true;
	uint8_t updatetime = 0;
	
	set_Modul_On(Wlan);
	_delay_ms(4000);

	if (!find_WiFly())
	{
		lcd_printp_at( 0, 5, PSTR("WiFly not found"), MNORMAL);
		set_Modul_On(USB);
		return (false);
	}
	else lcd_printp_at( 0, 5, PSTR("WiFly found"), MNORMAL);
	
	if (!setopt_RM(PSTR("set wlan join 1"), (char *)NULL, (char *)NULL,false)) //enables Command mode
	{
		lcd_printp_at( 0, 6, PSTR("F set wlan join"), MNORMAL);
		result = false;
	}
	
	if (!setSSID(Config.Wlan_SSID, false)) //set ssid
	{
		lcd_printp_at( 0, 6, PSTR("F set wlan ssid"), MNORMAL);
		result = false;
	}
	
    if (!setPassphrase(Config.Wlan_Password, false)) //set passphrase, least 8 characters long, max 32
    {
	    lcd_printp_at( 0, 6, PSTR("F apmode pass"), MNORMAL);
	    result = false;
    }
	
	if (!setopt_RM(PSTR("set ip dhcp 1"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ip dhcp"), MNORMAL);
		result = false;
	}
	
	if (!setopt_RM(PSTR("set ftp address 198.175.253.161"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ftp address"), MNORMAL);
		result = false;
	}
	
	if (!setopt_RM(PSTR("set ftp dir public"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ftp dir"), MNORMAL);
		result = false;
	}			
	
	if (!setopt_RM(PSTR("set ftp user roving"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ftp user"), MNORMAL);
		result = false;
	}
	
	if (!setopt_RM(PSTR("set ftp pass Pass123"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ftp pass"), MNORMAL);
		result = false;
	}		
	
	if (!setopt_RM(PSTR("set ftp filename wifly7-441.img"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ftp filename"), MNORMAL);
		result = false;
	}
	
	if (!setopt_RM(PSTR("set ftp timeout 200"), (char *)NULL, (char *)NULL,false))
	{
		lcd_printp_at( 0, 6, PSTR("F set ftp timeout"), MNORMAL);
		result = false;
	}
	
	_delay_ms(1000);
	save();
	_delay_ms(1000);
	reboot();
	for(updatetime = 0; updatetime < 30; updatetime++) _delay_ms(1000);
	send_P(PSTR("$$$"));
	inCommandMode = true;
	_delay_ms(1000);
	send_P(PSTR("ftp update\r"));
	for(updatetime = 0; updatetime < 30; updatetime++) _delay_ms(1000);
	save();
	_delay_ms(1000);
	setopt_RM(PSTR("factory RESET"), (char *)NULL, (char *)NULL,false);
	_delay_ms(1000);
	reboot();
	_delay_ms(1000);
	
	
	set_Modul_On(USB);
	if (!result) _delay_ms(4000);
	
	return (result);
}


/**
 * Set an option to an unsigned integer value
 * @param opt the set command to use (in Flash)
 * @param value the value to set
 * @param base the number format; DEC or HEX
 * @returns true on success, false on failure
 */
bool setopt(const char *opt, const uint32_t value, uint8_t base)
{
    char buf[11];
    simple_utoa(value, base, buf, sizeof(buf));
    return (setopt_RM(opt, buf,(char *)NULL,false));
}


/**
 * Set an option, confirm ok status
 * The value to set can either be in RAM or in Flash.
 * @param cmd the set command to use (string in Flash)
 * @param buf the value to set the option to (string in RAM). Set to NULL if not used.
 * @param buf_p the value to set the option to (string in Flash). Set to NULL if not used.
 * @param spacesub set to true to have spaces replaced with the
 *        current replacement character
 * @returns true on success, false on failure
 */
uint8_t setopt_RM(const char *cmd, const char *buf, const char *buf_P, uint8_t spacesub)
{
    char rbuf[16];
    bool res;

    if (!startCommand()) return (false);

    send_P(cmd);
    if (buf_P != NULL) 
	{
        sendc(' ');
        if (spacesub) 
		{
            const char *str = (const char *)buf_P;
            char ch;
            while ((ch = pgm_read_byte(str++)) != 0) 
			{
                if (ch == ' ') write(replaceChar);

				else write(ch);
            }
        } 
		else send_P((const char *)buf_P);

    } 
	else if (buf != NULL) 
	{
        sendc(' ');
        if (spacesub) 
		{
            const char *str = buf;
            char ch;
            /* spaces must be replaced */
            while ((ch = *str++) != 0) 
			{
                if (ch == ' ') write(replaceChar);
				else write(ch);
            }
        } 
		else sends(buf);
    }
    sendc('\r');
    res = getres(rbuf, sizeof(rbuf));
    getPrompt(0);
    finishCommand();
	
    return (res);
}


/**
 * Set an option to an unsigned integer value
 * @param opt the set command to use (in Flash)
 * @param value the value to set
 * @param base the number format; DEC or HEX
 * @returns true on success, false on failure
 */
uint8_t setopt_p(const char *opt, const uint32_t value, uint8_t base)
{
    char buf[11];
    simple_utoa(value, base, buf, sizeof(buf));
    return (setopt_RM(opt, buf,NULL,false));
}


/* Save current configuration */
bool save(void)
{
    bool res = false;

    if (!startCommand()) return (false);

    send_P(PSTR("save\r"));
    if (match_P(PSTR("Storing"),WIFLY_DEFAULT_TIMEOUT)) 
	{
        getPrompt(WIFLY_DEFAULT_TIMEOUT);
        res = true;
    }

    finishCommand();
    return (res);
}


/**
 * Read characters into the buffer until a carriage-return and newline is reached.
 * If the buffer is too small, the remaining characters in the line are discarded.
 * @param buf - the buffer to read into. If this is NULL then all characters in the line are discarded.
 * @param size - the size of the buffer (max number of characters it can store)
 * @param timeout - the number of milliseconds to wait for a new character to arrive
 * @returns the number of characters read into the buffer.
 * @retval 0 - timeout reading newline
 * @note The buffer will be null terminated, and in effect can hold size-1 characters.
 */
int gets(char *buf, int size, uint16_t timeout)
{
    char ch;
    int ind=0;

    while (readTimeout(&ch, timeout)) 
	{
        if (ch == '\r') 
		{
            readTimeout(&ch, timeout);
            if (ch == '\n') 
			{
                if (buf) buf[ind] = 0;

                return (ind);
            }
            if (buf) 
			{
                if (ind < (size-2)) 
				{
                    buf[ind++] = '\r';
                    buf[ind++] = ch;
                } 
				else if (ind < (size - 1)) buf[ind++] = '\r';
            }
        }
        /* Truncate to buffer size */
        if ((ind < (size-1)) && buf) buf[ind++] = ch;
    }

    if (buf) buf[ind] = 0;
	
    return (0);
}


//const char res_AOK[] PROGMEM = "AOK\r\n";
//const char res_ERR[] PROGMEM = "ERR: ";
/* Get the result from a set operation
 * Should be AOK or ERR
 */
bool getres(char *buf, int size)
{
    const char *setResult[] = {
        PSTR("ERR: "),
        PSTR("AOK\r\n")
    };
    int8_t res;

    res = multiMatch_P(setResult, 2,WIFLY_DEFAULT_TIMEOUT);

    if (res == 1) return (true);

	else if (res == 0) gets(buf, size,WIFLY_DEFAULT_TIMEOUT);

	else strncpy_P(buf, PSTR("<timeout>"), size); 

    return (false);
}


/* Finished with command */
bool finishCommand(void)
{
    if (--exitCommand == 0) return exitCommandMode();

    return (true);
}


/** Take the WiFly out of command mode */
bool exitCommandMode(void)
{
    if (!inCommandMode) return (true);

    send_P(PSTR("exit\r"));

    if (match_P(PSTR("EXIT\r\n"), WIFLY_DEFAULT_TIMEOUT)) 
	{
        inCommandMode = false;
        return (true);
    } 
	else return (false);
}


/** See if the prompt is somewhere in the string */
bool checkPrompt(const char *str)
{
    if (strstr(str, prompt) != NULL) return (true);
	else return (false);
}


/**
 * Read characters from the WiFly and match them against the set of
 * progmem strings. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until one of the strings is matched
 * or until no characters are received for the timeout duration.<br>
 * Example: res = multiMatch_P(500, 3, F("first="), F("second="), F("closed"));<br>
 * Will return 0 if "first=" is matched, 1 if "second=" is matched, 2 if "closed" is
 * matched, or -1 if nothing is matched and no data is received for 500 milliseconds.
 * @param timeout - fail if no data received for this period (in milliseconds).
 * @param count - the number of strings in the str array
 * @param ... A list of count progmem strings to match
 * @returns the index of the matching string
 * @retval -1 - no match found, timeout reached
 */
int multiMatch(uint16_t timeout, uint8_t count, ...)
{
    const char *str[20];
    int ind;
    va_list ap;
    va_start(ap, count);

    if (count > 20) count = 20;
    for (ind=0; ind<count; ind++) str[ind] = va_arg(ap, const char *);
    va_end(ap);

    return (multiMatch_P(str, count, timeout));
}


/**
 * Read characters from the WiFly and match them against the set of
 * progmem strings. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until one of the strings is matched
 * or until no characters are received for the timeout duration.
 * @param str - the array of progmem strings to match
 * @param count - the number of strings in the str array
 * @param timeout - fail if no data received for this period (in milliseconds).
 * @returns the index of the matching string
 * @retval -1 - no match found, timeout reached
 */
int8_t multiMatch_P(const char *str[], uint8_t count, uint16_t timeout)
{
    struct {
        bool active;
        const char *str;
    } match[count];
    char ch, ch_P;
    uint8_t ind;

    for (ind=0; ind<count; ind++) 
	{
        match[ind].active = false;
        match[ind].str = str[ind];
    }

    while (readTimeout(&ch,timeout)) 
	{
        for (ind=0; ind<count; ind++) 
		{
            ch_P = pgm_read_byte(match[ind].str);
			
            if (ch == ch_P) match[ind].str++;
			else 
			{
                match[ind].str = str[ind]; /* Restart match */
                if (ch == pgm_read_byte(match[ind].str)) match[ind].str++; /* unmatched character might be the start of the string */
            }
			
            if (pgm_read_byte(match[ind].str) == '\0') return ind; /* Got a match */
        }
    }
	
    return (-1);
}


/**
 * Read characters from the WiFly and match them against the
 * flash string. Ignore any leading characters that don't match. Keep
 * reading, discarding the input, until the string is matched
 * or until no characters are received for the timeout duration.
 * @param str The string to match (in flash memory)
 * @timeout fail if no data received for this period (in milliseconds).
 * @retval true - a match was found
 * @retval false - no match found, timeout reached
 */
bool match(const char *str, uint16_t timeout)
{
    return (match_P((const char *)str, timeout));
}


/* Get the value of an option */
char *getopt(int opt, char *buf, int size)
{
    if (startCommand()) 
	{
        send_P(requests[opt].req);

		if (match_P(requests[opt].resp, 500))
		{
            gets(buf, size,WIFLY_DEFAULT_TIMEOUT);
            getPrompt(0);
            finishCommand();
			
            return (buf);
        }
        finishCommand();
	}
	
    return ((char *)"<error>");
}


bool disableDHCP()
{
	return (setDHCP(0));
}


bool enableDHCP()
{
	return (setDHCP(1));
}


bool setSSID(const char *buf, bool select)
{
    if(select == true) return (setopt_RM(PSTR("set apmode ssid"), (char *)buf, (char *)NULL, true));
	else return (setopt_RM(PSTR("set wlan ssid"), (char *)buf, (char *)NULL, true));
}


bool setDHCP(const uint8_t mode)
{
	char buf[2];

	if (mode > 9) {
		return false;
	}

	buf[0] = '0' + mode;
	buf[1] = 0;

	return (setopt_RM(PSTR("set ip dhcp"),(char *)buf, (char *)NULL, false));
//  return setopt_RM (PSTR("set ip dhcp"),buf,NULL, false);
}

/** Set the WiFi channel.
 * @param channel the wifi channel from 0 to 13. 0 means auto channel scan.
 * @returns true if successful, else false.
 */
bool setChannel(uint8_t channel)
{
    if (channel > 13) channel = 13;

    return (setopt_p(PSTR("set wlan chan"), channel, DEC));
}


/** Set WEP key */
bool setKey(const char *buf)
{
	if ((buf[1] == 'x') || (buf[1] == 'X')) 
	{
        /* Skip over the 0x leader */
        buf += 2;
    }

    return (setopt_RM(PSTR("set wlan key"), buf,NULL,false));
}


/**
 * Set WPA passphrase.
 * Spaces are automatically replaced with the current space substitution
 * character ('$' is the default).
 * @Note: If your passphrase contains a '$' then use setSpaceReplace() to
 *        change the replacement character to something you're not using.
 */
bool setPassphrase(const char *buf, bool select)
{
	if(select == true) return (setopt_RM(PSTR("set apmode pass"), buf, NULL, true));
    else return (setopt_RM(PSTR("set wlan phrase"), buf, NULL, true));
}


/** Hide passphrase and key */
bool hide(void)
{
	return setopt_RM(PSTR("set wlan hide 1"), (char *)NULL,NULL,true);
}


/** Get Version */
char * getVersion(char *buf, int size)
{
	char *version;
	
	set_Modul_On(Wlan);
	_delay_ms(4000);

	if (!find_WiFly())
	{
		lcd_printp_at( 0, 5, PSTR("WiFly not found"), MNORMAL);
	}
	else lcd_printp_at( 0, 5, PSTR("WiFly found"), MNORMAL);
	
	version = getopt(WIFLY_GET_VERSION, buf, size);
	
	set_Modul_On(USB);
	
	return (version);
}


bool setJoin(uint8_t join)
{
	return (setopt(PSTR("set wlan join"), join,DEC));
}


//**
// * Create an Adhoc WiFi network.
// * The WiFly is assigned IP address 169.254.1.1.
// * @param ssid the SSID to use for the network
// * @param channel the WiFi channel to use; 1 to 13.
// * @retval true - successfully create Ad Hoc network
// * @retval false - failed
// * @note the WiFly is rebooted as the final step of this command.
// */
uint8_t createAdhocNetwork(const char *ssid, uint8_t channel)
{
	Result=0;

	if (!startCommand()) Result = 1;
	if (Result==0) if(!setDHCP(WIFLY_DHCP_MODE_OFF)) Result = 2;
	if (Result==0) if(!setIP(Config.Wlan_IP)) Result = 3;
	if (Result==0) if(!setNetmask(Config.Wlan_Netmask)) Result = 4;
	if (Result==0) if(!setJoin(WIFLY_WLAN_JOIN_ADHOC)) Result = 5;
	if (Result==0) if(!setSSID(ssid, false)) Result = 6;
	if (Result==0) if(!setChannel(channel)) Result = 7;
	if (!save()) Result = 8;
//  _delay_ms(500);
	if (!finishCommand()) Result = 9;
	reboot();
	return Result;
}


/**
 * Set the WiFly's IP address.
 * @param buf IPv4 address in a null-terminated string.
 *            e.g "192.168.1.10"
 * @retval true address set
 * @retval false failed to set address
 */
bool setIP(const char *buf)
{
    return setopt_RM(PSTR("set ip address"),(char *) buf,NULL,false);
}


/** Reboots the WiFly.
* @note Depending on the shield, this may also reboot the Arduino.
*/
bool reboot(void)
{
    if (!startCommand()) return (false);

    send_P(PSTR("reboot\r"));
    if (!match_P(PSTR("*Reboot*"), WIFLY_DEFAULT_TIMEOUT)) 
	{
		finishCommand();
		
		return (false);
    }

    _delay_ms(5000);
    inCommandMode = false;
    exitCommand = 0;
	
    return (true);
}


bool setHostIP(const char *buf)
{
	return setopt_RM(PSTR("set ip host"), buf,NULL,false);
}


bool setNetmask(const char *buf)
{
	return setopt_RM(PSTR("set ip netmask"), buf,NULL,false);
}


bool setGateway(const char *buf)
{
	return setopt_RM(PSTR("set ip gateway"), buf,NULL,false);
}


bool setBaud(uint8_t baud)
{
    char buf[16];
	
	switch (baud) 
	{

      case Baud_2400   : strcpy_P( buf, PSTR("2400")); break;
      case Baud_4800   : strcpy_P( buf, PSTR("4800")); break;
      case Baud_9600   : strcpy_P( buf, PSTR("9600")); break;
      case Baud_19200  : strcpy_P( buf, PSTR("19200")); break;
      case Baud_38400  : strcpy_P( buf, PSTR("38400")); break;
      case Baud_57600  : strcpy_P( buf, PSTR("57600")); break;
      case Baud_115200 : strcpy_P( buf, PSTR("115200")); break;
      break;
	}
	
    /* Go into command mode, since "set uart instant" will exit command mode */
    startCommand();
	
    if (setopt_RM(PSTR("set uart baud"), buf,NULL,false)) 
	{
        setopt_RM(PSTR("set uart instant"), buf,NULL,false);
        SetBaudUart1(Config.PKT_Baudrate);
		
        return (true);
    }
	
    SetBaudUart1(Config.PKT_Baudrate);
    return (true);
}


bool find_WiFly(void)  // search Wifly with all Baudrates
{
	uint8_t Result = false;

	lcd_printp_at( 0, 2, PSTR("search WiFly"), MNORMAL);
	Old_Baudrate = Config.PKT_Baudrate;
	
	if(enterCommandMode())
	{ 
		lcd_printp_at( 0, 4, PSTR("search 57600 Baud"), MNORMAL);
		Result = true;
		_delay_ms(1000);
	}
	
	if (!Result)
    {
		SetBaudUart1(Baud_9600);
        lcd_printp_at( 0, 4, PSTR("search 9600 Baud"), MNORMAL);
        if(enterCommandMode()) Result = true;
		_delay_ms(1000);
	}

	if (!Result)
    {
		SetBaudUart1(Baud_115200);
        lcd_printp_at( 0, 4, PSTR("search 115200 Baud"), MNORMAL);
        if(enterCommandMode()) Result = true;
		_delay_ms(1000);
	}
	
	if (!Result)
	{
		SetBaudUart1(Baud_38400);
		lcd_printp_at( 0, 4, PSTR("search 38400 Baud"), MNORMAL);
		if(enterCommandMode()) Result = true;
		_delay_ms(1000);
	}
	
	if (!Result)
	{
		SetBaudUart1(Baud_19200);
		lcd_printp_at( 0, 4, PSTR("search 19200 Baud"), MNORMAL);
		if(enterCommandMode()) Result = true;
		_delay_ms(1000);
	}
	
	if (!Result)
	{
		SetBaudUart1(Baud_4800);
		lcd_printp_at( 0, 4, PSTR("search 4800 Baud"), MNORMAL);
		if(enterCommandMode()) Result = true;
		_delay_ms(1000);
	}
	
	if (!Result)
	{
		SetBaudUart1(Baud_2400);
		lcd_printp_at( 0, 4, PSTR("search 2400 Baud"), MNORMAL);
		if(enterCommandMode()) Result = true;
		_delay_ms(1000);
	}			

    if (!Result)
	{
		lcd_printp_at( 0, 4, PSTR("WiFly find error"), MNORMAL);
//		lcd_printp_at( 0, 4, PSTR("use config with PC"), MNORMAL);
		Result = false;
		_delay_ms(1000);
    }

	return (Result);
}


#endif

